import os
import sys

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), "../..")))

host = "127.0.0.1"
acl_agent_port = 9999
translator_agent_port = 9998
test_client_port = 9997
